@@ -128,19 +128,28 @@ def _parse_age_re(age, unit, tp="date"):
 
     pattern = rf"(\d+\.?\d*)\s*({pat_un}s?)"
     matchstr = re.match(pattern, age, flags=re.I)
+    swap_flag = False
     if matchstr is None:
         # checking pattern with "unit" word
         pattern_unit = rf"(\d+\.?\d*)\s*units?:?\s*({pat_un}s?)"
         matchstr = re.match(pattern_unit, age, flags=re.I)
         if matchstr is None:
-            return age, None
-
-    if "." in matchstr.group(1):
-        qty = float(matchstr.group(1))
+            # checking patter with swapped order
+            pattern = rf"({pat_un}s?)\s*(\d+\.?\d*)"
+            matchstr = re.match(pattern, age, flags=re.I)
+            swap_flag = True
+            if matchstr is None:
+                return age, None
+    if swap_flag:
+        qty = matchstr.group(3)
+    else:
+        qty = matchstr.group(1)
+    if "." in qty:
+        qty = float(qty)
         if int(qty) == qty:
             qty = int(qty)
     else:
-        qty = int(matchstr.group(1))
+        qty = int(qty)
     age_rem = age.replace(matchstr.group(0), "")
     age_rem = age_rem.strip()
     return age_rem, f"{qty}{unit}"
@@ -191,6 +200,15 @@ def parse_age(age):
         raise ValueError("age is empty")
 
     age_orig = age
+
+    if "gestation" in age.lower():
+        pattern_time = "^gest[a-z]*"
+        matchstr = re.match(pattern_time, age, flags=re.I)
+        age = age.replace(matchstr.group(0), "")
+        ref = "Gestational"
+    else:
+        ref = "Birth"
+
     age = age.strip()
 
     if age[0] == "P":
@@ -213,20 +231,24 @@ def parse_age(age):
             elif part_f:
                 date_f = ["P", part_f]
 
-        time_f = []
-        for un in ["H", "M", "S"]:
-            if not age:
-                break
-            age, part_f = _parse_age_re(age, un, tp="time")
-            if part_f and time_f:
-                time_f.append(part_f)
-            elif part_f:
-                time_f = ["T", part_f]
-        # trying to find formats 00:00:00 for time
-        if not time_f:
-            age, time_f = _parse_hours_format(age)
+        if ref == "Birth":
+            time_f = []
+            for un in ["H", "M", "S"]:
+                if not age:
+                    break
+                age, part_f = _parse_age_re(age, un, tp="time")
+                if part_f and time_f:
+                    time_f.append(part_f)
+                elif part_f:
+                    time_f = ["T", part_f]
+            # trying to find formats 00:00:00 for time
+            if not time_f:
+                age, time_f = _parse_hours_format(age)
 
-        age_f = date_f + time_f
+            age_f = date_f + time_f
+        elif ref == "Gestational":
+            # ignore time formats for Gestational (unless it is needed in the future)
+            age_f = date_f
         if set(age) - {" ", ".", ",", ":", ";"}:
             raise ValueError(
                 f"not able to parse the age: {age_orig}, no rules to convert: {age}"
@@ -238,8 +260,7 @@ def parse_age(age):
             f"decimal fraction allowed in the lowest order part only,"
             f" but {age} was received "
         )
-
-    return "".join(age_f)
+    return "".join(age_f), ref
 
 
 def extract_age(metadata):
@@ -247,15 +268,18 @@ def extract_age(metadata):
         dob = ensure_datetime(metadata["date_of_birth"])
         start = ensure_datetime(metadata["session_start_time"])
     except (KeyError, TypeError, ValueError):
-        try:
-            duration = parse_age(metadata["age"])
-        except (KeyError, TypeError, ValueError):
+        if metadata.get("age") is not None:
+            duration, ref = parse_age(metadata["age"])
+        else:
             return ...
     else:
-        duration = timedelta2duration(start - dob)
+        duration, ref = timedelta2duration(start - dob), "Birth"
     return models.PropertyValue(
         value=duration,
         unitText="ISO-8601 duration",
+        valueReference=models.PropertyValue(
+            value=getattr(models.AgeReferenceType, f"{ref}Reference")
+        ),
     )
 
 
@@ -323,7 +347,13 @@ def extract_species(metadata):
             value = "Human"
         elif "norvegicus" in value:
             value_id = "http://purl.obolibrary.org/obo/NCBITaxon_10116"
-            value = "Brown rat"
+            value = "Rat; Norway rat; rats; Brown rat"
+        elif "rattus rattus" in value:
+            value_id = "http://purl.obolibrary.org/obo/NCBITaxon_10117"
+            value = "Black rat; Roof rat; House rat"
+        elif "rat" in value:
+            value_id = "http://purl.obolibrary.org/obo/NCBITaxon_10116"
+            value = "Rat; Norway rat; rats; Brown rat"
         elif "mulatta" in value or "rhesus" in value:
             value_id = "http://purl.obolibrary.org/obo/NCBITaxon_9544"
             value = "Rhesus monkey"
@@ -337,7 +367,10 @@ def extract_species(metadata):
             value_id = value
             value = None
         else:
-            raise ValueError(f"Cannot interpret species field: {value}")
+            raise ValueError(
+                f"Cannot interpret species field: {value}. Please "
+                "contact help@dandiarchive.org to add your species."
+            )
         return models.SpeciesType(identifier=value_id, name=value.capitalize())
     else:
         return ...
@@ -411,13 +444,17 @@ def extract_session(metadata: dict) -> list:
     for val in probe_ids:
         probes.append(models.Equipment(identifier=f"probe:{val}", name="Ecephys Probe"))
     probes = probes or None
-    if (metadata.get("session_id") or probes) is None:
+    session_id = None
+    if metadata.get("session_id") is not None:
+        session_id = str(metadata["session_id"])
+    if (session_id or metadata.get("session_start_time") or probes) is None:
         return None
     return [
         models.Session(
-            identifier=metadata.get("session_id"),
-            name=metadata.get("session_id", "Acquisition session"),
+            identifier=session_id,
+            name=session_id or "Acquisition session",
             description=metadata.get("session_description"),
+            startDate=metadata.get("session_start_time"),
             used=probes,
         )
     ]
@@ -660,6 +697,30 @@ def process_ndtypes(asset, nd_types):
     return asset
 
 
+def get_asset_metadata(
+    filepath, relpath, digest=None, digest_type=None, allow_any_path=True
+) -> models.BareAsset:
+    metadata = None
+    if op.splitext(filepath)[1] == ".nwb":
+        try:
+            metadata = nwb2asset(filepath, digest=digest, digest_type=digest_type)
+        except Exception as e:
+            lgr.warning(
+                "Failed to extract NWB metadata from %s: %s: %s",
+                filepath,
+                type(e).__name__,
+                str(e),
+            )
+            if not allow_any_path:
+                raise
+    if metadata is None:
+        metadata = get_default_metadata(
+            filepath, digest=digest, digest_type=digest_type
+        )
+    metadata.path = str(relpath)
+    return metadata
+
+
 def nwb2asset(
     nwb_path, digest=None, digest_type=None, schema_version=None
 ) -> models.BareAsset:
