@@ -5,10 +5,12 @@ try:
 except ImportError:
     # TODO - remove whenever python >= 3.8
     from importlib_metadata import version as importlib_version
+
 import inspect
 import io
 import itertools
 import logging
+from mimetypes import guess_type
 import os
 import os.path as op
 from pathlib import Path
@@ -26,13 +28,12 @@ import ruamel.yaml
 from semantic_version import Version
 import tenacity
 
-from .consts import dandi_instance, known_instances, known_instances_rev
-from .exceptions import BadCliVersionError, CliVersionTooOldError
-
 #
 # Additional handlers
 #
 from . import __version__, get_logger
+from .consts import DandiInstance, known_instances, known_instances_rev
+from .exceptions import BadCliVersionError, CliVersionTooOldError
 
 lgr = get_logger()
 
@@ -109,7 +110,7 @@ def get_utcnow_datetime(microseconds=True):
         return ret.replace(microsecond=0)
 
 
-def is_same_time(*times, tollerance=1e-6, strip_tzinfo=False):
+def is_same_time(*times, tolerance=1e-6, strip_tzinfo=False):
     """Helper to do comparison between time points
 
     Time zone information gets stripped
@@ -118,8 +119,8 @@ def is_same_time(*times, tollerance=1e-6, strip_tzinfo=False):
 
     Parameters
     ----------
-    tollerance: float, optional
-      Seconds of difference between times to tollerate.  By default difference
+    tolerance: float, optional
+      Seconds of difference between times to tolerate.  By default difference
       up to a microsecond is ok
     """
     assert len(times) >= 2
@@ -130,10 +131,10 @@ def is_same_time(*times, tollerance=1e-6, strip_tzinfo=False):
     ]
 
     # we need to have all pairs
-    tollerance_dt = datetime.timedelta(seconds=tollerance)
+    tolerance_dt = datetime.timedelta(seconds=tolerance)
     return all(
         # if we subtract from smaller - we get negative days etc
-        (t1 - t2 if t1 > t2 else t2 - t1) <= tollerance_dt
+        (t1 - t2 if t1 > t2 else t2 - t1) <= tolerance_dt
         for (t1, t2) in itertools.combinations(norm_times, 2)
     )
 
@@ -212,16 +213,6 @@ def flattened(it):
     return list(flatten(it))
 
 
-def updated(d, update):
-    """Return a copy of the input with the 'update'
-
-    Primarily for updating dictionaries
-    """
-    d = d.copy()
-    d.update(update)
-    return d
-
-
 def remap_dict(rec, revmapping):
     """Remap nested dicts according to mapping
 
@@ -360,14 +351,19 @@ def find_files(
         paths = (op.join(dirpath, name) for name in names)
         for path in filter(re.compile(regex).search, paths):
             if not exclude_path(path):
-                yield path
+                if op.islink(path) and op.isdir(path):
+                    lgr.warning(
+                        "%s: Ignoring unsupported symbolic link to directory", path
+                    )
+                else:
+                    yield path
 
 
 _cp_supports_reflink = None
 
 
 def copy_file(src, dst):
-    """ Copy file from src to dst """
+    """Copy file from src to dst"""
     global _cp_supports_reflink
     if _cp_supports_reflink is None:
         r = subprocess.run(
@@ -472,18 +468,6 @@ def _get_normalized_paths(path, prefix):
     return path, prefix
 
 
-def path_startswith(path, prefix):
-    """Return True if path starts with prefix path
-
-    Parameters
-    ----------
-    path: str
-    prefix: str
-    """
-    path, prefix = _get_normalized_paths(path, prefix)
-    return path.startswith(prefix)
-
-
 def path_is_subpath(path, prefix):
     """Return True if path is a subpath of prefix
 
@@ -498,14 +482,6 @@ def path_is_subpath(path, prefix):
     return (len(prefix) < len(path)) and path.startswith(prefix)
 
 
-def safe_call(func, path, default=None):
-    try:
-        return func(path)
-    except Exception as exc:
-        lgr.debug("Call to %s on %s failed: %s", func.__name__, path, exc)
-        return default
-
-
 def shortened_repr(value, length=30):
     try:
         if hasattr(value, "__repr__") and (value.__repr__ is not object.__repr__):
@@ -618,55 +594,26 @@ def get_instance(dandi_instance_id):
         raise CliVersionTooOldError(our_version, minversion, bad_versions)
     if our_version in bad_versions:
         raise BadCliVersionError(our_version, minversion, bad_versions)
-    return dandi_instance(
-        metadata_version=0,
-        girder=server_info["services"]["girder"]["url"],
-        gui=server_info["services"]["webui"]["url"],
-        redirector=redirector_url,
-        api=server_info["services"]["api"]["url"],
-    )
-
-
-TITLE_CASE_LOWER = {
-    "a",
-    "an",
-    "and",
-    "as",
-    "but",
-    "by",
-    "for",
-    "in",
-    "nor",
-    "of",
-    "on",
-    "or",
-    "the",
-    "to",
-    "with",
-}
-
-
-def name2title(name):
-    # For use in autopopulating the titles of model schema fields
-    words = []
-    for w in split_camel_case(name):
-        w = w.lower()
-        if w == "id" or w == "url":
-            w = w.upper()
-        elif not words or w not in TITLE_CASE_LOWER:
-            w = w.capitalize()
-        words.append(w)
-    return " ".join(words)
-
-
-def split_camel_case(s):
-    last_start = 0
-    # Don't split apart "ID":
-    for m in re.finditer(r"(?<=I)[A-CE-Z]|(?<=[^I])[A-Z]", s):
-        yield s[last_start : m.start()]
-        last_start = m.start()
-    if last_start < len(s):
-        yield s[last_start:]
+    # note: service: url, not a full record
+    services = {
+        name: (rec or {}).get(
+            "url"
+        )  # note: somehow was ending up with {"girder": None}
+        for name, rec in server_info.get("services", {}).items()
+    }
+    if services.get("api"):
+        return DandiInstance(
+            gui=services.get("webui"),
+            redirector=redirector_url,
+            api=services.get("api"),
+        )
+    else:
+        raise RuntimeError(
+            "redirector's server-info returned unknown set of services keys: "
+            + ", ".join(
+                k for k, v in server_info.get("services", {}).items() if v is not None
+            )
+        )
 
 
 def try_multiple(ntrials, retry, base):
@@ -693,13 +640,14 @@ def is_url(s):
 
     TODO: redo
     """
-    return s.lower().startswith(("http://", "https://", "dandi://"))
+    return s.lower().startswith(("http://", "https://", "dandi:"))
+    # Slashes are not required after "dandi:" so as to support "DANDI:<id>"
 
 
 def get_module_version(module: Union[str, types.ModuleType]) -> Optional[str]:
     """Return version of the module
 
-    Return module's `__version__` and if present, or use importlib
+    Return module's `__version__` if present, or use importlib
     to get version.
 
     Returns
@@ -708,13 +656,14 @@ def get_module_version(module: Union[str, types.ModuleType]) -> Optional[str]:
     """
     if isinstance(module, str):
         mod_name = module
-        if module not in sys.modules:
-            return None
-        module = sys.modules[module]
+        module = sys.modules.get(module)
     else:
         mod_name = module.__name__.split(".", 1)[0]
 
-    version = getattr(module, "__version__", None)
+    if module is not None:
+        version = getattr(module, "__version__", None)
+    else:
+        version = None
     if version is None:
         # Let's use the standard Python mechanism if underlying module
         # did not provide __version__
@@ -723,3 +672,89 @@ def get_module_version(module: Union[str, types.ModuleType]) -> Optional[str]:
         except Exception as exc:
             lgr.debug("Failed to determine version of the %s: %s", mod_name, exc)
     return version
+
+
+def pluralize(n: int, word: str, plural: Optional[str] = None) -> str:
+    if n == 1:
+        return f"{n} {word}"
+    else:
+        if plural is None:
+            plural = word + "s"
+        return f"{n} {plural}"
+
+
+def abbrev_prompt(msg: str, *options: str) -> str:
+    """
+    Prompt the user to input one of several options, which can be entered as
+    either a whole word or the first letter of a word.  All input is handled
+    case-insensitively.  Returns the complete word corresponding to the input,
+    lowercased.
+
+    For example, ``abbrev_prompt("Delete assets?", "yes", "no", "list")``
+    prompts the user with the message ``Delete assets? ([y]es/[n]o/[l]ist): ``
+    and accepts as input ``y`, ``yes``, ``n``, ``no``, ``l``, and ``list``.
+    """
+    options_map = {}
+    optstrs = []
+    for opt in options:
+        opt = opt.lower()
+        if opt in options_map:
+            raise ValueError(f"Repeated option: {opt}")
+        elif opt[0] in options_map:
+            raise ValueError(f"Repeated abbreviated option: {opt[0]}")
+        options_map[opt] = opt
+        options_map[opt[0]] = opt
+        optstrs.append(f"[{opt[0]}]{opt[1:]}")
+    msg += " (" + "/".join(optstrs) + "): "
+    while True:
+        answer = input(msg).lower()
+        if answer in options_map:
+            return options_map[answer]
+
+
+def get_mime_type(filename: str, strict: bool = False) -> str:
+    """
+    Like `mimetypes.guess_type()`, except that if the file is compressed, the
+    MIME type for the compression is returned.  Also, the default return value
+    is now ``'application/octet-stream'`` instead of `None`.
+    """
+    mtype, encoding = guess_type(filename, strict)
+    if encoding is None:
+        return mtype or "application/octet-stream"
+    elif encoding == "gzip":
+        # application/gzip is defined by RFC 6713
+        return "application/gzip"
+        # There is also a "+gzip" MIME structured syntax suffix defined by RFC
+        # 8460; exactly when can that be used?
+        # return mtype + '+gzip'
+    else:
+        return "application/x-" + encoding
+
+
+def check_dandi_version():
+    if os.environ.get("DANDI_NO_ET"):
+        return
+    try:
+        import etelemetry
+
+        try:
+            etelemetry.check_available_version(
+                "dandi/dandi-cli", __version__, lgr=lgr, raise_exception=True
+            )
+        except etelemetry.client.BadVersionError:
+            # note: SystemExit is based of BaseException, so is not Exception
+            raise SystemExit(
+                "DANDI CLI has detected that you are using a version that is known to "
+                "contain bugs, is incompatible with our current data archive, or has "
+                "other significant performance limitations. "
+                "To continue using DANDI CLI, please upgrade your dandi client to a newer "
+                "version (e.g., using pip install --upgrade dandi if you installed using pip). "
+                "If you have any issues, please contact the DANDI "
+                "helpdesk at https://github.com/dandi/helpdesk/issues/new/choose ."
+            )
+    except Exception as exc:
+        lgr.warning(
+            "Failed to check for a more recent version available with etelemetry: %s",
+            exc,
+        )
+    os.environ["DANDI_NO_ET"] = "1"
